!!ARBvp1.0
# Copyright Brad Anderson 2004

# Passthrough Vertex Program by James A. McCombe, Tue Jun 17 2003
#
# This program acts as a simple passthrough.  It simply transforms the vertex position
# from model space into eye space and passes through the vertex color and first texture
# coordinate

ATTRIB iPos		= vertex.position;
ATTRIB iNormalModel	= vertex.normal;

OUTPUT oPos		= result.position;

PARAM  mvp[4]        = { state.matrix.mvp };
PARAM  mv[4]	     = { state.matrix.modelview };
PARAM  mvit[4]	     = { state.matrix.modelview.invtrans };
PARAM  lit_mvi[4]    = { state.matrix.modelview.invtrans };

PARAM  texmat0[4]    = { state.matrix.texture[0] };
#PARAM  texmat1[4]    = { state.matrix.texture[1] };
#PARAM  texmat2[4]    = { state.matrix.texture[2] };
#PARAM  texmat3[4]    = { state.matrix.texture[3] };
#PARAM  lightPos      = program.env[0];
PARAM lightDirNonNormalized = program.env[0];
PARAM eyePos = state.matrix.modelview.inverse.row[3];

PARAM  zero      = {0, 0, 0, 0};

TEMP  eyeNormal, lightDirModel, t0, t1, halfAngle, dotProds;


TEMP	eyeVertex, vertToLight, vertToEye;



# Transform the vertex by the modelview/projection matrix
    DP4    oPos.x, mvp[0], iPos;
    DP4    oPos.y, mvp[1], iPos;
    DP4    oPos.z, mvp[2], iPos;
    DP4    oPos.w, mvp[3], iPos;
    
    
# Transform the vertex by the modelview/projection matrix
    DP4    eyeVertex.x, mv[0], iPos;
    DP4    eyeVertex.y, mv[1], iPos;
    DP4    eyeVertex.z, mv[2], iPos;
    DP4    eyeVertex.w, mv[3], iPos;

# Transform the light position into model space, then normalize
    TEMP normalModel, invtEyeNormal;
	DP3 normalModel.w, iNormalModel, iNormalModel;
	RSQ normalModel.w, normalModel.w;
	MUL normalModel, iNormalModel, normalModel.w;    # t0 now contains the normalized
    DP3 eyeNormal.x, mvit[0], normalModel;
    DP3 eyeNormal.y, mvit[1], normalModel;
    DP3 eyeNormal.z, mvit[2], normalModel;

# Transform the light position into model space, then normalize
    TEMP lightDirection;
	DP3 lightDirection.w, lightDirNonNormalized, lightDirNonNormalized;
	RSQ lightDirection.w, lightDirection.w;
	MUL lightDirection, lightDirNonNormalized, lightDirection.w;    # t0 now contains the normalized
    DP3 lightDirModel.x, mvit[0], lightDirection;
    DP3 lightDirModel.y, mvit[1], lightDirection;
    DP3 lightDirModel.z, mvit[2], lightDirection;

#compute the vertex to light vector
    MOV vertToLight, lightDirModel;

#compute vertex to eye
    DP3 vertToEye.w, eyeVertex, eyeVertex;
    RSQ vertToEye.w, vertToEye.w;
    MUL vertToEye, -eyeVertex, vertToEye.w;


#normalize the light direction, normalizing the normal is a per fragment function


#compute the diffuse intensity
TEMP diffuse;
DP3  diffuse.x, eyeNormal, lightDirModel;
MAX  diffuse, diffuse.xxxx, zero;



# normalized vector from fragment to eye
# half vector is normalize(light vector + the eye vector)
    ADD   halfAngle, vertToEye, vertToLight;
    DP3   halfAngle.w, halfAngle, halfAngle;
    RSQ   halfAngle.w, halfAngle.w;
    MUL   halfAngle, halfAngle, halfAngle.w;
#specular  = Ks * light color * pow(max(dot(N,H),0),  shininess);






# Pass the color and texture coordinate through
MUL    t0, t0, vertex.color;
MOV    result.color, t0;

DP4    result.texcoord[0].x, texmat0[0], vertex.texcoord[0];
DP4    result.texcoord[0].y, texmat0[1], vertex.texcoord[0];

MOV    result.texcoord[1], eyeNormal;
MOV    result.texcoord[2], lightDirModel;


MOV    result.texcoord[3], halfAngle;
MOV    result.texcoord[4], vertToEye;
#MOV    result.texcoord[5], iPos;

END
